# -*- coding: utf-8 -*-
"""
smash.core
~~~~~~~~~~

Core functionality for Smash.
"""

import os
import shelve


class Smash(object):
    """Main class for Smash."""

    def __init__(self):
        self.db_path = os.path.join(os.path.expanduser('~'), 
                                    '.smash', 'smash.db') 
        try:
            self.db = shelve.open(self.db_path)
        except:
            self.db = None 

        self.move = {
            'down' : ':',
            'none' : '+',
            'up'   : ']',
        }  

    def close(self):
        """Close the database object."""
        self.db.close()

    def set_custom_mov(self, new_mov):
        """Set custom movements."""
        movs = new_mov.split(',')
        if len(movs) == 3 and all([len(c) == 1 for c in movs]):
            self.move['down'] = movs[0]
            self.move['none'] = movs[1]
            self.move['up'] = movs[2]
            return True
        else:
            return False

    def save(self, name, dirs):
        """Save the directory tree string with the given name."""
        if self.db is not None:
            self.db[name] = dirs
            return True

    def load(self, name):
        """Load the directory tree string with the given name."""
        if self.db is not None and name in self.db:
            return self.db[name]

    def remove(self, name):
        """Remove the directory tree string with the given name."""
        if self.db is not None:
            if name in self.db:
                del self.db[name]
                return True

    def list_all(self):
        """Return all stored directory trees."""
        return [(name, dirs) for name, dirs in self.db.items()]

    def tokens(self, str):
        """Return all tokens from the given directory tree string.

        A token is:
        - a word whose characters are not in the attribute self.move
        - each character contained in the attribute self.move
        """
        token = ""

        for i in range(len(str) + 1):
            if i < len(str):
                is_mov = str[i] in self.move.values()
                end_string = i == len(str) - 1

                movs = [self.move['down'], self.move['none']]
                if ((is_mov and not token) or (is_mov and end_string)
                    or (str[i] in movs and token in self.move.values())
                    or (is_mov and token in movs)):
                    raise ValueError
                elif is_mov or token in self.move.values():
                    yield token
                    token = str[i]
                else:
                    token += str[i]
            else:
                yield token

    def make_paths(self, topdir, str):
        """Return all paths from the given directory tree string."""
        stack = [topdir]
        paths = []

        for token in self.tokens(str):
            if token not in self.move.values():
                path = os.path.join(stack[-1], token)
                paths.append(path)
            elif token == self.move['down']:
                stack.append(paths[-1])
            elif token == self.move['up'] and len(stack) > 1:
                stack.pop()

        return paths

    def make_dirtree(self, topdir, str):
        """Creates all the directories from the given directory tree string."""
        created = []
        not_created = []

        for p in self.make_paths(topdir, str):
            try:
                os.makedirs(p)
                created.append(p)
            except OSError:
                not_created.append(p)

        return created, not_created


    # functions for printing the preview of the directory tree
    # ========================================================================

    def has_sibling(self, rest):
        """Check if the previous token (folder) has a sibling.
        
           A sibling of a folder is a folder at the same level.
           This function helps to know when to insert a vertical connector
           when printing the directory tree in the preview.
        """
        sum = 0
        for i, t in enumerate(rest):
            if t == self.move['down']:
                sum += 1
            elif t == self.move['up']:
                sum -= 1
            if sum < 0:
                return False
            if sum == 0:
                return True

        return False

    def make_spacer(self, vert_links, width):
        """Construct the spacing for indent the folders when printing them"""
        spacing = ""
        for i in range(width):
            if i in vert_links:
                spacing += '│'
            else:
                spacing += ' '

        return spacing

    def print_dirtree(self, topdir, str, colors):
        """Print the directory tree."""
        stack_ws = []
        default_spacing = 4
        spacing = 0
        vert_links = set()

        ts = [t for t in self.tokens(str)]
        print('.')
        for i, token in enumerate(ts):

            if token not in self.move.values():
                spacer = self.make_spacer(vert_links, spacing)
                curr_path = self.make_paths(topdir, "".join(ts[:i+1]))[-1]

                color = ""
                if os.path.exists(curr_path):
                    color = colors.red

                conn = ""
                if i + 1 < len(ts) and self.has_sibling(ts[i + 1:]):
                    vert_links.add(spacing)
                    conn = "├── "
                else:
                    if spacing in vert_links:
                        vert_links.remove(spacing)
                    conn = "└── "

                print(spacer + conn + color + token + colors.default)

            elif token == self.move['down']:
                stack_ws.append(spacing)
                spacing += default_spacing

            elif token == self.move['up'] and len(stack_ws) > 0:
                spacing = stack_ws.pop()
